@@ -78,6 +78,7 @@ class Rollouts(object):
 
         self.laststate = laststate
         self.current_price = float(laststate['features']['price'])
+        self.current_hashrate = float(laststate['features']['hashrate'])
         self.n_rollouts = n_rollouts
         self.gamma_params = np.array([48] * n_rollouts)
         self.models = models
@@ -87,8 +88,11 @@ class Rollouts(object):
 
         f = self.laststate['features']
         for k in f:
-            if 'height' not in k:
-                f[k] = f[k] + zero_array
+            f[k] = f[k] + zero_array
+
+        # Include height as features:
+        f['start_height'] = self.laststate['phase'] * 48
+        f['end_height']   = f['start_height'] + 48
 
         self.timeline = [f]
 
@@ -112,7 +116,10 @@ class Rollouts(object):
         The first of these is 1-dimensional; all others are 2-dimensional.
         '''
 
-        return np.stack([t[key] for t in self.timeline], axis=0)
+        if key in self.timeline[-1]:
+            return np.stack([t[key] for t in self.timeline], axis=0)
+        elif ('log_' + key) in self.timeline[-1]:
+            return np.exp(self['log_' + key])
 
     def run_model(self, modelname, currstate):
 
