@@ -1,3 +1,6 @@
+import numpy as np
+import pandas as pd
+
 from .history import *
 from .rollouts import extract_design_matrix, Rollouts
 from .parsing import parse_date, to_unix_time
@@ -34,6 +37,53 @@ class Simulator(object):
 
         self.models = {}
 
+    def plot_history(self, minx=None, maxx=None):
+
+        if maxx is None:
+            maxx = 48 * len(self.history)
+
+        if minx is None:
+            minx = max(0, maxx - 210000)
+
+        from matplotlib import pyplot as plt
+
+        if 'hashrate' not in self.models:
+            self.train_hashrate_model()
+
+        hashes_per_block = np.exp(self.history['zero_bits'] * np.log(2)).values
+        block_heights = (self.history['phase'].values + 1) * 48
+
+        df = pd.DataFrame({
+            'empirical hashrate': self.history['hashrate'].values * 1.0e-18,
+            'mining difficulty': hashes_per_block / 600e18,
+            'price': np.nan
+        }, index=block_heights)
+
+        ax = df.loc[minx:maxx].plot(
+            figsize=(12, 5),
+            style=['y', 'b', 'r'],
+            xlim=(minx, maxx),
+            fontsize=14)
+
+        plt.ylabel('global hashrate (EH/s)', fontsize=18)
+        plt.xlabel('block height', fontsize=18)
+
+        leg = ['empirical hashrate', 'mining difficulty', 'price', '3rd halving']
+        for i in range(1, 34):
+            hbh = 210000 * i
+            if (minx <= hbh <= maxx):
+                plt.axvline(hbh, color='grey', linestyle='--')
+                ordinal_suffix = ('st' if i in [1, 21, 31] else ('nd' if i in [2, 22, 32] else ('rd' if i in [3, 23, 33] else 'th')));
+                leg.append('%d%s halving' % (i, ordinal_suffix))
+        plt.legend(leg, fontsize=16)
+
+        df['price'] = self.history['price'].values
+        df['price'].loc[minx:maxx].plot(ax=ax, style='r', secondary_y=True, alpha=0.4, fontsize=14)
+        plt.ylabel('bitcoin price (USD)', fontsize=18)
+
+        return ax
+
+
     def run_rollouts(self, n_rollouts, n_iters=0, until_height=None, until_time=None):
 
         if 'hashrate' not in self.models:
